/************************************************************************
 *
 * Copyright 2024 J David Eisenberg

 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0.
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************************/
import java.io.File;
import java.util.Scanner;
import java.util.GregorianCalendar;

import org.odftoolkit.odfdom.doc.OdfSpreadsheetDocument;
import org.odftoolkit.odfdom.dom.OdfContentDom;
import org.odftoolkit.odfdom.dom.OdfStylesDom;

import org.odftoolkit.odfdom.incubator.doc.number.OdfNumberDateStyle;
import org.odftoolkit.odfdom.incubator.doc.number.OdfNumberStyle;
import org.odftoolkit.odfdom.incubator.doc.number.OdfNumberTimeStyle;

import org.odftoolkit.odfdom.incubator.doc.office.OdfOfficeAutomaticStyles;
import org.odftoolkit.odfdom.incubator.doc.office.OdfOfficeStyles;

import org.odftoolkit.odfdom.dom.element.style.StyleParagraphPropertiesElement;

import org.odftoolkit.odfdom.incubator.doc.style.OdfStyle;

import org.odftoolkit.odfdom.dom.element.style.StyleTextPropertiesElement;

import org.odftoolkit.odfdom.doc.table.OdfTable;
import org.odftoolkit.odfdom.doc.table.OdfTableCell;
import org.odftoolkit.odfdom.doc.table.OdfTableColumn;
import org.odftoolkit.odfdom.doc.table.OdfTableRow;

import org.odftoolkit.odfdom.dom.attribute.office.OfficeValueTypeAttribute;
import org.odftoolkit.odfdom.dom.element.OdfStyleBase;
import org.odftoolkit.odfdom.dom.element.table.TableTableCellElementBase;
import org.odftoolkit.odfdom.dom.style.OdfStyleFamily;

/**
 *
 * @author J David Eisenberg
 */
public class WeatherSpreadsheet {

    String inputFileName;
    String outputFileName;

    OdfSpreadsheetDocument outputDocument;
    OdfContentDom contentDom;    // the document object model for content.xml
    OdfStylesDom stylesDom;    // the document object model for styles.xml

    // the office:automatic-styles element in content.xml
    OdfOfficeAutomaticStyles automaticStyles;

    // the office:styles element in styles.xml
    OdfOfficeStyles officeStyles;

    // These are the styles we will create
    OdfStyle rowStyle;
    OdfStyle columnStyle;
    OdfStyle headingStyle;
    OdfStyle weatherDateStyle;
    OdfStyle weatherTimeStyle;
    OdfStyle weatherTemperatureStyle;

    // the office:spreadsheet element in the content.xml file
    OdfTable spreadsheetTable;

    public void run(String[] args) {
        if (args.length == 2) {
            inputFileName = args[0];
            outputFileName = args[1];

            setupOutputDocument();

            if (outputDocument != null) {
                addAutomaticStyles();
                processInputDocument();
                saveOutputDocument();
            }
        } else {
            System.err.println("Usage: WeatherSpreadsheet infile outfile");
        }
    }

    void setupOutputDocument() {
        try {
            outputDocument =
                OdfSpreadsheetDocument.newSpreadsheetDocument();
            contentDom = outputDocument.getContentDom();
            stylesDom = outputDocument.getStylesDom();
            automaticStyles = contentDom.getOrCreateAutomaticStyles();
            officeStyles = outputDocument.getOrCreateDocumentStyles();
        } catch (Exception e) {
            System.err.println("Unable to create output file.");
            System.err.println(e.getMessage());
            outputDocument = null;
        }
    }

    void setFontWeight(OdfStyleBase style, String value) {
        style.setProperty(StyleTextPropertiesElement.FontWeight, value);
        style.setProperty(StyleTextPropertiesElement.FontWeightAsian, value);
        style.setProperty(StyleTextPropertiesElement.FontWeightComplex, value);
    }

    void addAutomaticStyles() {
        // bold centered cells (for first row)
        headingStyle = automaticStyles.newStyle(OdfStyleFamily.TableCell);
        headingStyle.setProperty(StyleParagraphPropertiesElement.TextAlign, "center");
        setFontWeight(headingStyle, "bold");

        // Create the date, time, and temperature styles and add them.
        // The null in OdfNumberDateStyle means "use default calendar system"
        OdfNumberDateStyle dateStyle = new OdfNumberDateStyle(contentDom,
            "yyyy-MM-dd", "weatherDateStyle", null);
        OdfNumberTimeStyle timeStyle = new OdfNumberTimeStyle(contentDom,
            "hh:mm", "weatherTimeStyle");
        OdfNumberStyle numberStyle = new OdfNumberStyle(contentDom,
            "#0.00", "weatherTemperatureStyle");

        automaticStyles.appendChild(dateStyle);
        automaticStyles.appendChild(timeStyle);
        automaticStyles.appendChild(numberStyle);

        // cell style for Date cells
        weatherDateStyle = automaticStyles.newStyle(OdfStyleFamily.TableCell);
        weatherDateStyle.setStyleDataStyleNameAttribute("weatherDateStyle");

        // and for time cells
        weatherTimeStyle = automaticStyles.newStyle(OdfStyleFamily.TableCell);
        weatherTimeStyle.setStyleDataStyleNameAttribute("weatherTimeStyle");

        // and for the temperatures
        weatherTemperatureStyle = automaticStyles.newStyle(OdfStyleFamily.TableCell);
        weatherTemperatureStyle.setStyleDataStyleNameAttribute("weatherTemperatureStyle");
        weatherTemperatureStyle.setProperty(StyleParagraphPropertiesElement.TextAlign, "right");
     }

    void processInputDocument() {
        Scanner input;        // for reading the file
        String data;          // holds one line of the file
        String[] info;        // holds the split-up data

        TableTableCellElementBase cellElement;

        spreadsheetTable = outputDocument.getSpreadsheetTables().get(0);
        spreadsheetTable.getColumnByIndex(0).setDefaultCellStyle(weatherDateStyle);
        spreadsheetTable.getColumnByIndex(1).setDefaultCellStyle(weatherTimeStyle);
        spreadsheetTable.getColumnByIndex(2).setDefaultCellStyle(weatherTemperatureStyle);

        for (int i = 0; i < 3; i++) {
            spreadsheetTable.getColumnByIndex(i).setWidth(25); // 2.5 cm
        }

        // fill in the header row
        OdfTableRow row = spreadsheetTable.getRowByIndex(0);
        // row.setDefaultCellStyle(headingStyle);
        row.getCellByIndex(0).setStringValue("Date");
        row.getCellByIndex(1).setStringValue("Time");
        row.getCellByIndex(2).setStringValue("\u00b0C");

        int rowNumber = 1;
        try {
            input = new Scanner(new File(inputFileName));
            while (input.hasNextLine()) {
                data = input.nextLine();
                info = data.split(",");
                /* 0 = date, 1 = time, 2 = temperatue in degrees C */
                row = spreadsheetTable.getRowByIndex(rowNumber);
                row.getCellByIndex(0).setDateValue(toCalendar(info[0], info[1]));
                row.getCellByIndex(1).setTimeValue(toCalendar(info[0], info[1]));
                row.getCellByIndex(2).setDoubleValue(Double.parseDouble(info[2]));
                row.setHeight(5, true);
                rowNumber++;
            }
            input.close();

            row = spreadsheetTable.getRowByIndex(rowNumber + 1);
            row.getCellByIndex(1).setStringValue("Avg. Temp.");
            row.getCellByIndex(1).getOdfElement().setStyleName(headingStyle.getStyleNameAttribute());


            cellElement = row.getCellByIndex(2).getOdfElement();
            cellElement.setOfficeValueTypeAttribute(
                OfficeValueTypeAttribute.Value.FLOAT.toString());
            cellElement.setTableFormulaAttribute("of:=AVERAGE([.C2:.C" + (rowNumber) + "]");

            row = spreadsheetTable.getRowByIndex(0);
            for (int i = 0; i < 3; i++) {
                row.getCellByIndex(i).getOdfElement().setStyleName(headingStyle.getStyleNameAttribute());
            }

        } catch (Exception e) {
            System.err.println("Cannot process " + inputFileName);
            e.printStackTrace();
        }
    }

    private GregorianCalendar toCalendar(String date, String time) {
        String[] timeParts = time.split(":");
        String[] dateParts = date.split("-");
        int[] timeInts = {Integer.parseInt(timeParts[0]), Integer.parseInt(timeParts[1]),
            Integer.parseInt(timeParts[2])};
        /* months start at zero; that's why we subtract 1 from the month */
        int[] dateInts = {Integer.parseInt(dateParts[0]), Integer.parseInt(dateParts[1]) - 1,
            Integer.parseInt(dateParts[2])};


        GregorianCalendar result = new GregorianCalendar(
            dateInts[0], dateInts[1], dateInts[2],
            timeInts[0], timeInts[1], timeInts[2]);
        return result;
    }

    void saveOutputDocument() {
        try {
            outputDocument.save(outputFileName);
        } catch (Exception e) {
            System.err.println("Unable to save document.");
            System.err.println(e.getMessage());
        }
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        WeatherSpreadsheet app = new WeatherSpreadsheet();
        app.run(args);
    }
}
